@@ -91,30 +91,17 @@ heistErrMsg msg = do
 tellSpliceError :: Monad m => Text -> HeistT n m ()
 tellSpliceError msg = do
     hs <- getHS
+    node <- getParamNode
     let spliceError = SpliceError
                       { spliceHistory = _splicePath hs
                       , spliceTemplateFile = _curTemplateFile hs
                       , visibleSplices = Map.keys $ _compiledSpliceMap hs
+                      , contextNode = node
                       , spliceMsg = msg
                       }
     modifyHS (\hs' -> hs { _spliceErrors = spliceError : _spliceErrors hs' })
 
 
-------------------------------------------------------------------------------
--- | Transform a SpliceError record to a Text message.
-spliceErrorText :: SpliceError -> Text
-spliceErrorText (SpliceError hist tf splices msg) =
-    (maybe "" ((`mappend` ": ") . T.pack) tf) `T.append` msg `T.append`
-    foldr (\(_, tf', tag) -> (("\n   ... via " `T.append`
-                               (maybe "" ((`mappend` ": ") . T.pack) tf')
-                               `T.append` tag) `T.append`)) T.empty hist
-    `T.append`
-    if null splices
-      then T.empty
-      else "\nBound splices:" `T.append`
-         foldl (\x y -> x `T.append` " " `T.append` y) T.empty splices
-
-
 ------------------------------------------------------------------------------
 -- | Function for showing a TPath.
 showTPath :: TPath -> String
