@@ -14,6 +14,7 @@ module Heist.Compiled.Internal where
 import           Blaze.ByteString.Builder
 import           Blaze.ByteString.Builder.Char.Utf8
 import           Control.Arrow
+import           Control.Exception
 import           Control.Monad
 import           Control.Monad.RWS.Strict
 import           Control.Monad.State.Strict
@@ -290,14 +291,33 @@ runNode node = localParamNode (const node) $ do
     let pre = _splicePrefix hs
     let hasPrefix = (T.isPrefixOf pre `fmap` X.tagName node) == Just True
     when (not (T.null pre) && hasPrefix) incNamespacedTags
-    isStatic <- subtreeIsStatic node
-    markup <- getsHS _curMarkup
-    if isStatic
-      then return $! yieldPure $! renderFragment markup [parseAttrs node]
-      else localHS (\hs' -> hs' {_splicePath =
-                                 (_curContext hs', _curTemplateFile hs',
-                                  X.elementTag node):(_splicePath hs')}) $
-           compileNode node
+    hs' <- getHS
+    -- Plain rethrows for CompileException to avoid multiple annotations.
+    (res, hs'') <- liftIO $ catches (compileIO hs')
+                     [ Handler (\(ex :: CompileException) -> throwIO ex)
+                     , Handler (\(ex :: SomeException) -> handleError ex hs')]
+    putHS hs''
+    return res
+  where
+    localSplicePath =
+        localHS (\hs -> hs {_splicePath = (_curContext hs,
+                                           _curTemplateFile hs,
+                                           X.elementTag node):
+                                          (_splicePath hs)})
+    compileIO hs = runHeistT compile node hs
+    compile = do
+        isStatic <- subtreeIsStatic node
+        dl <- compile' isStatic
+        liftIO $ evaluate $ DL.fromList $! consolidate dl
+    compile' True = do
+        markup <- getsHS _curMarkup
+        return $! yieldPure $! renderFragment markup [parseAttrs node]
+    compile' False = localSplicePath $ compileNode node
+    handleError ex hs = do
+        errs <- evalHeistT (do localSplicePath $ tellSpliceError $ T.pack $
+                                 "Exception in splice compile: " ++ show ex
+                               getsHS _spliceErrors) node hs
+        throwIO $ CompileException ex errs
 
 
 parseAttrs :: X.Node -> X.Node
@@ -606,13 +626,14 @@ renderTemplate hs nm =
 -- | Looks up a compiled template and returns a compiled splice.
 callTemplate :: Monad n
              => ByteString
-             -> HeistT n IO (DList (Chunk n))
+             -> Splice n
 callTemplate nm = do
     hs <- getHS
-    runNodeList $ maybe (error err) (X.docContent . dfDoc . fst) $
-      lookupTemplate nm hs _templateMap
+    maybe (error err) call $ lookupTemplate nm hs _templateMap
   where
     err = "callTemplate: "++(T.unpack $ T.decodeUtf8 nm)++(" does not exist")
+    call (df,_) = localHS (\hs' -> hs' {_curTemplateFile = dfFile df}) $
+                    runNodeList $ X.docContent $ dfDoc df
 
 
 interpret :: Monad n => DList (Chunk n) -> n Builder
@@ -735,42 +756,37 @@ deferMany f getItems = do
 ------------------------------------------------------------------------------
 -- | Saves the results of a runtme computation in a 'Promise' so they don't
 -- get recalculated if used more than once.
+--
+-- Note that this is just a specialized version of function application ($)
+-- done for the side effect in runtime splice.
+defer :: Monad n
+      => (RuntimeSplice n a -> Splice n)
+      -> RuntimeSplice n a -> Splice n
+defer pf n = do
+    p2 <- newEmptyPromise
+    let action = yieldRuntimeEffect $ putPromise p2 =<< n
+    res <- pf $ getPromise p2
+    return $ action `mappend` res
+
+
+------------------------------------------------------------------------------
+-- | A version of defer which applies a function on the runtime value.
 deferMap :: Monad n
          => (a -> RuntimeSplice n b)
          -> (RuntimeSplice n b -> Splice n)
          -> RuntimeSplice n a -> Splice n
-deferMap f pf n = do
-    p2 <- newEmptyPromise
-    let action = yieldRuntimeEffect $ putPromise p2 =<< f =<< n
-    res <- pf $ getPromise p2
-    return $ action `mappend` res
+deferMap f pf n = defer pf $ f =<< n
 
 
 ------------------------------------------------------------------------------
 -- | Like deferMap, but only runs the result if a Maybe function of the
 -- runtime value returns Just.  If it returns Nothing, then no output is
 -- generated.
---
--- This is a good example of how to do more complex flow control with
--- promises.  The generalization of this abstraction is too complex to be
--- distilled to elegant high-level combinators.  If you need to implement your
--- own special flow control, then you should use functions from the
--- `Heist.Compiled.LowLevel` module similarly to how it is done in the
--- implementation of this function.
 mayDeferMap :: Monad n
             => (a -> RuntimeSplice n (Maybe b))
             -> (RuntimeSplice n b -> Splice n)
             -> RuntimeSplice n a -> Splice n
-mayDeferMap f pf n = do
-    p2 <- newEmptyPromise
-    action <- pf $ getPromise p2
-    return $ yieldRuntime $ do
-        mb <- f =<< n
-        case mb of
-          Nothing -> return mempty
-          Just b -> do
-            putPromise p2 b
-            codeGen action
+mayDeferMap f pf n = deferMany pf $ f =<< n
 
 
 ------------------------------------------------------------------------------
